## MGF goodness-of-fit test statistic for bivariate skew-normal distribution
##
## functions:
##         gof.snr ... R implementation
##         gof.snc ... wrapper for C implementation in Linux (file "gofskewn.c" 
##                     is needed and a shared library "gofskewn.so" has to be
##                     created: "R CMD SHLIB gofskewn.c")
##
## input:
##         x ... matrix (nx2), n observations of a bivariate random vector
##         d1 .. parameter delta1
##         d2 .. parameter delta2
##         om .. parameter omega
##         a ... parameter a of the goodness-of-fit test
## output:
##         value of the gof test statistic based on mgf
##
## for more details see Meintanis and Hlavka (2009): Goodness-of-fit tests
## for bivariate and multivariate skew-normal distributions.
##
##
## R implementation
gof.snr=function (x,d1,d2,om,a=2) {
  n=nrow(x)
  x1jk=outer(x[,1],x[,1],"+")
  x2jk=outer(x[,2],x[,2],"+")
  p1jk=outer(x[,1],x[,1],"*")
  p2jk=outer(x[,2],x[,2],"*")
  p3jk=outer(x[,1],x[,2],"*")
  k1=d1-om*d2
  k2=d2-om*d1
  i0x1=exp((x1jk^2)/(4*a))*sqrt(pi/a)
  i0x2=exp((x2jk^2)/(4*a))*sqrt(pi/a)
  i1x1=i0x1*x1jk/(2*a)
  i1x2=i0x2*x2jk/(2*a)
  i2x1=i0x1*(1+((x1jk^2)/(2*a)))/(2*a)
  i2x2=i0x2*(1+((x2jk^2)/(2*a)))/(2*a)
  t1=sum(((d2^2)*p1jk+(d1^2)*p2jk-2*d1*d2*p3jk)*i0x1*i0x2)/n
  t2=sum((k2^2)*i2x1*i0x2+(k1^2)*i2x2*i0x1-2*k1*k2*i1x1*i1x2)/n
  t3=2*sum((d2*x[,1]-d1*x[,2])*k1*i1x2*i0x1+(d1*x[,2]-d2*x[,1])*k2*i1x1*i0x2)/n
  t=t1+t2+t3
  return(t)
}
##
## wrapper for the C code "gofskewn.c"
gof.snc <- function (x,d1,d2,om,a=2)
.C("gofskewn",x1=as.double(x[,1]),x2=as.double(x[,2]),n=as.integer(nrow(x)),as.double(d1),as.double(d2),as.double(om),as.double(a),t=double(1))$t
dyn.load("./gofskewn.so")
##
## a simple example to verify that both programs give identical results
x.t=cbind(c(-1,-0.4,1.2,0.2,2.4),c(-3,-0.27,1.14,0.93,0.14))
t1=gof.snr(x.t,d1=0.5,d2=0.5,om=0.25,a=4)
t2=gof.snc(x.t,d1=0.5,d2=0.5,om=0.25,a=4)
##
## let us test the speed of the algorithm
n=100
x.tn=matrix(rnorm(2*n),ncol=2)
system.time(for (i in 1:100) t1n=gof.snr(x.tn,d1=0.5,d2=0.5,om=0.25,a=4))
system.time(for (i in 1:100) t2n=gof.snc(x.tn,d1=0.5,d2=0.5,om=0.25,a=4))
##
